# -*- coding: utf-8 -*-

import os
import sys
import xbmc
import xbmcaddon
import xbmcgui
import xbmcplugin
import urllib
import re
from config import ADDON, ADDON_PATH, RUTOR_URL, ADDON_URL, mode, ADDON_ID, ADDON_PARAMS, addon_name, kodi_notification, USE_PROXY, logger, EXCHANGE_TORRENT_INFO, TURBO_MODE, ADDON_PROFILE, SORT_ORDER, EXT_TITLE
from rutor import RuTor, get_torrent_files, MEDIA_TYPES
import database
from download_torrent_info import send_download_torrent_info_notification
from exchange_torrent_info import ExchangeTorrentInfo
import tools

db = database.Database()


def addon_url_builder(params={}):
    return '%s?%s' % (ADDON_URL, '&'.join(map(lambda x: '%s=%s' % (str(x[0]), urllib.quote_plus(str(x[1]))), params.items())))


ACTION_PREVIOUS_MENU = 10  # По умолчанию - ESC
ACTION_NAV_BACK = 92  # По умолчанию - Backspace


def pool_get_info(t_url):
    rutor = RuTor(RUTOR_URL)
    return rutor.get_info(t_url)


def item_list_builder(items):
    """
    форматирование текста в коди https://kodi.wiki/view/Label_Formatting
    infoLabels http://mirrors.kodi.tv/docs/python-docs/16.x-jarvis/xbmcgui.html#ListItem-setInfo
    """
    items = filter(lambda i: not re.findall('( PC )|( MP3 )|MP3|JPG', i['name']), items)
    rutor = RuTor(RUTOR_URL)
    for i in items:
        i['url'] = rutor.link(i['url'])  # По возможности стараться работать с путем без домена, т.к. домены меняются из-за блокировки
    quality_as_data = ADDON.getSetting("quality_as_date") == 'true'
    do_download_torrents_info = False
    exchange_torrent_info = {}
    if not TURBO_MODE and EXCHANGE_TORRENT_INFO:  # Спрашиваем у обменника информацию о торрентах
        exchange_id_list = []
        for i in items:
            info = db.torrent_info_get(i['url'], create_download_task=False)
            if info is None:
                exchange_id_list.append(database.normalize_torrent_info_url(i['url']))
        if len(exchange_id_list) > 0:
            exchange_torrent_info = ExchangeTorrentInfo().get_info(exchange_id_list)
    # Сортировка
    if SORT_ORDER == 1:  # По качеству
        items.sort(lambda b, a: cmp(tools.parce_quality(a['name']), tools.parce_quality(b['name'])))
    elif SORT_ORDER == 2:  # По размеру
        items.sort(lambda b, a: cmp(tools.size_to_byte(a.get('size', '')), tools.size_to_byte(b.get('size', ''))))
    elif SORT_ORDER == 3:  # По имени
        items.sort(lambda a, b: cmp(a['name'], b['name']))
    elif SORT_ORDER == 4:  # Качество/Размер
        items.sort(lambda b, a: cmp(
            (tools.parce_quality(a['name']), tools.size_to_byte(a.get('size', ''))),
            (tools.parce_quality(b['name']), tools.size_to_byte(b.get('size', ''))),
        ))
    elif SORT_ORDER == 5:  # Качество/Имя
        items.sort(lambda b, a: cmp(
            (tools.parce_quality(a['name']), b['name']),
            (tools.parce_quality(b['name']), a['name']),
        ))

    # Формирование списка
    for i in items:
        name = i['name']
        if len(name) > 80:
            if name.find('(') > -1:
                name = name.split('(')[0]
        info = None
        if not TURBO_MODE:
            t_id = database.normalize_torrent_info_url(i['url'])
            if t_id in exchange_torrent_info:
                info = exchange_torrent_info[t_id]
                db.torrent_info_update(i['url'], info)
            else:
                info = db.torrent_info_get(i['url'])
        if info is not None:
            info_labels = info['kodi_info']
            if i['name'] != info['header']:
                db.torrent_info_del(i['url'])
                db.torrent_info_get(i['url'], create_download_task=False)
                do_download_torrents_info = True
                logger.debug('Описание торрента изменилось, перекачать, было "%s" стало "%s"' % (info['header'], i['name']))
        else:
            info_labels = {}
            do_download_torrents_info = True
        info_l = info if info is not None else {}
        if 'size' in i:
            size = ' [%s %s] ' % i['size']
        else:
            size = ''
        title = info_labels.get('title', name)
        if EXT_TITLE:
            title = '%s | %s | %s' % (
                str(i.get('seeds', info_l.get('seeds', 0))).center(4),
                str('%s %s' % i.get('size', tuple(info_l.get('size', ('', '  '))))).center(7),
                title
            )
        list_item = xbmcgui.ListItem(title)
        info_labels['plot'] = i['name'] + size + '[CR][CR]' + info_labels.get('plot', '')
        if quality_as_data:
            quality = tools.parce_quality(i['name'])
            if quality is not None:
                info_labels['year'] = quality
            else:
                if 'year' in info_labels:
                    del info_labels['year']
        list_item.setInfo(type="Video", infoLabels=info_labels)
        if info is not None:
            art = {}
            if info['thumbnail']:
                art['thumb'] = info['thumbnail']
            else:
                if info['scr_images']:
                    art['thumb'] = info['scr_images'].pop()
            if info['scr_images']:
                art['fanart'] = info['scr_images'].pop()
            if art:
                list_item.setArt(art)
        xbmcplugin.addDirectoryItem(ADDON_ID, addon_url_builder({
            'mode': 'torrent_files',
            'torrent': i['torrent'],
            't_url': RuTor.unlink(i['url']),
            'name': i['name'],
        }), list_item, isFolder=True)
        if do_download_torrents_info and not TURBO_MODE:
            send_download_torrent_info_notification()


def mode_search():
    if mode == 'search':
        input_text = xbmc.Keyboard()
        input_text.setHeading('Поиск:')
        input_text.doModal()
        if not input_text.isConfirmed():
            mode_index()
            return
        text = input_text.getText()
        if len(text) < 3:
            return
        db.db_info['last_search'] = text
        db.save_db_info()
    else:
        text = db.db_info['last_search']
    rutor = RuTor(RUTOR_URL)
    try:
        res = rutor.search(text)
    except Exception, e:
        kodi_notification('Ошибка в поиске %s' % e)
        return
    if res:
        item_list_builder(res)
        xbmcplugin.setPluginCategory(ADDON_ID, 'Поиск: %s' % text)
        xbmcplugin.endOfDirectory(ADDON_ID)


def mode_clean_cache():
    db.clean_cache(full=True)
    xbmcgui.Dialog().notification(addon_name, 'Кэш очищен!', icon=os.path.join(ADDON_PATH, 'images', 'clean.png'))


def mode_index():
    list_item = xbmcgui.ListItem('Поиск')
    list_item.setArt({'icon': os.path.join(ADDON_PATH, 'images', 'search.png')})
    xbmcplugin.addDirectoryItem(ADDON_ID, ADDON_URL + '?mode=search', list_item, isFolder=True)

    if 'last_search' in db.db_info:
        list_item = xbmcgui.ListItem('Поиск: [COLOR FF0000AA]%s[/COLOR]' % db.db_info['last_search'])
        list_item.setArt({'icon': os.path.join(ADDON_PATH, 'images', 'search.png')})
        xbmcplugin.addDirectoryItem(ADDON_ID, ADDON_URL + '?mode=last_search', list_item, isFolder=True)

    list_item = xbmcgui.ListItem('[COLOR FF00FF00]Топ[/COLOR]')
    list_item.setArt({'icon': os.path.join(ADDON_PATH, 'images', 'top.png')})
    xbmcplugin.addDirectoryItem(ADDON_ID, ADDON_URL + '?mode=top', list_item, isFolder=True)

    list_item = xbmcgui.ListItem('[COLOR FFe55ea2]Избранное[/COLOR]')
    list_item.setArt({'icon': os.path.join(ADDON_PATH, 'images', 'favorite.png')})
    xbmcplugin.addDirectoryItem(ADDON_ID, ADDON_URL + '?mode=favorites', list_item, isFolder=True)

    list_item = xbmcgui.ListItem('Очистить кэш')
    list_item.setArt({'icon': os.path.join(ADDON_PATH, 'images', 'clean.png')})
    xbmcplugin.addDirectoryItem(ADDON_ID, ADDON_URL + '?mode=clean_cache', list_item)

    if USE_PROXY == 2:
        list_item = xbmcgui.ListItem('Создать прокси лист')
        xbmcplugin.addDirectoryItem(ADDON_ID, ADDON_URL + '?mode=generate_proxy_list', list_item)

    # list_item = xbmcgui.ListItem('Test gui')
    # xbmcplugin.addDirectoryItem(addon_id, addon_url + '?mode=test_gui', list_item)

    xbmcplugin.setPluginCategory(ADDON_ID, addon_name)
    xbmcplugin.endOfDirectory(ADDON_ID)


def mode_torrent_files():
    rutor = RuTor(RUTOR_URL)
    torrent = urllib.unquote_plus(ADDON_PARAMS.get('torrent'))
    unlinked_t_url = rutor.unlink(urllib.unquote_plus(ADDON_PARAMS.get('t_url')))
    t_url = rutor.link(unlinked_t_url)
    name = urllib.unquote_plus(ADDON_PARAMS.get('name'))
    list_item = xbmcgui.ListItem('[B]Связанные раздачи[/B]')
    xbmcplugin.addDirectoryItem(ADDON_ID, addon_url_builder({'mode': 'linked_torrents', 't_url': t_url}), list_item, isFolder=True)
    if not TURBO_MODE:
        try:  # Тест, не изменился ли торрент или информация о нем
            real_info = rutor.get_info(t_url)
            db_info = db.torrent_info_get(t_url)
            if db_info is None or real_info['header'] != db_info['header'] or torrent != real_info['torrent'] or real_info['header'] != name:
                db.torrent_info_update(t_url, real_info)
                if EXCHANGE_TORRENT_INFO:
                    ExchangeTorrentInfo().set_info(database.normalize_torrent_info_url(t_url), real_info)
                torrent = real_info['torrent']
                if db.favorites_in(unlinked_t_url):
                    db.favorites_add({
                        'url': unlinked_t_url,
                        'torrent': real_info['torrent'],
                        'name': real_info['header'],
                    })
        except:
            logger.exception('torrent files exception', exc_info=True)

    torrent_files = filter(lambda i: i['path'][-1].split('.')[-1].lower() in MEDIA_TYPES, get_torrent_files(rutor.link(torrent)))
    torrent_files.sort(lambda a, b: cmp(a['path'][-1], b['path'][-1]))
    if db.favorites_in(unlinked_t_url):
        list_item = xbmcgui.ListItem('[B]Удалить из избранного[/B]')
        xbmcplugin.addDirectoryItem(ADDON_ID, addon_url_builder({'mode': 'del_from_favorites', 't_url': unlinked_t_url}), list_item)
    else:
        list_item = xbmcgui.ListItem('[B]Добавить в избранное[/B]')
        xbmcplugin.addDirectoryItem(ADDON_ID, addon_url_builder({
            'mode': 'add_to_favorites',
            't_url': unlinked_t_url,
            'torrent': torrent,
            'name': name,
        }), list_item)
    for i in torrent_files:
        list_item = xbmcgui.ListItem(i['path'][-1])
        list_item.setProperty('IsPlayable', 'true')
        xbmcplugin.addDirectoryItem(ADDON_ID, addon_url_builder({'mode': 'play', 'torrent': torrent, 'findex': i['findex'], 'fn': i['path'][-1]}), list_item)
    xbmcplugin.setPluginCategory(ADDON_ID, addon_name)
    xbmcplugin.endOfDirectory(ADDON_ID)


def mode_linked_torrents():
    rutor = RuTor(RUTOR_URL)
    t_url = urllib.unquote_plus(ADDON_PARAMS.get('t_url'))
    try:
        linked_list = rutor.get_linked_torrents(t_url)
        item_list_builder(linked_list)
    except:
        pass
    xbmcplugin.setPluginCategory(ADDON_ID, addon_name)
    xbmcplugin.endOfDirectory(ADDON_ID)


def mode_play():
    rutor = RuTor(RUTOR_URL)
    torrent = rutor.link(urllib.unquote_plus(ADDON_PARAMS.get('torrent')))
    findex = int(urllib.unquote_plus(ADDON_PARAMS.get('findex')))
    fn = urllib.unquote_plus(ADDON_PARAMS.get('fn'))
    from p2p import play_torrent
    engine = int(ADDON.getSetting("p2p_engine"))
    play_torrent(engine, torrent, findex, fn)


TOP_MENU = {
    1: {'capt': 'Топ торренты за последние 24 часа', 'url': 'top', 'table': 1},
    2: {'capt': 'Зарубежные фильмы', 'url': 'kino', 'table': 2},
    3: {'capt': 'Наши фильмы', 'url': 'nashe_kino', 'table': 2},
    4: {'capt': 'Научно-популярные фильмы', 'url': 'nauchno_popularnoe', 'table': 2},
    5: {'capt': 'Сериалы', 'url': 'seriali', 'table': 2},
    6: {'capt': 'Телевизор', 'url': 'tv', 'table': 2},
    7: {'capt': 'Аниме', 'url': 'anime', 'table': 2},
    8: {'capt': 'Юмор', 'url': 'jumor', 'table': 2},
}


def mode_top():
    submenu = int(ADDON_PARAMS.get('submenu', 0))
    if submenu == 0:
        for k, v in TOP_MENU.iteritems():
            list_item = xbmcgui.ListItem(v['capt'])
            xbmcplugin.addDirectoryItem(ADDON_ID, addon_url_builder({'mode': 'top', 'submenu': k}), list_item, isFolder=True)
    else:
        rutor = RuTor(RUTOR_URL)
        item_list_builder(rutor.get_index(TOP_MENU[submenu]['url'], TOP_MENU[submenu]['table']))
    xbmcplugin.setPluginCategory(ADDON_ID, addon_name)
    xbmcplugin.endOfDirectory(ADDON_ID)


def mode_test_gui():
    send_download_torrent_info_notification()
    kodi_notification('download_torrent_info')
    pass


def mode_generate_proxy_list():
    from proxy_list_generator import generate
    pd = xbmcgui.DialogProgress()
    pd.create('Генерация прокси листа')
    msg_buf = []
    for p, s in generate(ADDON_PROFILE):
        msg_buf.append(s)
        pd.update(p, '[CR]'.join(msg_buf[-4:]))
    pd.close()
    del pd


def mode_add_to_favorites():
    item = {
        'torrent': urllib.unquote_plus(ADDON_PARAMS.get('torrent')),
        'url': RuTor.unlink(urllib.unquote_plus(ADDON_PARAMS.get('t_url'))),
        'name': urllib.unquote_plus(ADDON_PARAMS.get('name')),
    }
    db.favorites_add(item)
    kodi_notification('Добавлено')


def mode_del_from_favorites():
    t_url = RuTor.unlink(urllib.unquote_plus(ADDON_PARAMS.get('t_url')))
    db.favorites_del(t_url)
    kodi_notification('Удалено')


def mode_favorites():
    item_list_builder(list(db.get_favorites()))
    xbmcplugin.setPluginCategory(ADDON_ID, addon_name)
    xbmcplugin.endOfDirectory(ADDON_ID)


def main():
    router = {
        'index': mode_index,
        'search': mode_search,
        'last_search': mode_search,
        'clean_cache': mode_clean_cache,
        'torrent_files': mode_torrent_files,
        'play': mode_play,
        'top': mode_top,
        'test_gui': mode_test_gui,
        'generate_proxy_list': mode_generate_proxy_list,
        'linked_torrents': mode_linked_torrents,
        'add_to_favorites': mode_add_to_favorites,
        'del_from_favorites': mode_del_from_favorites,
        'favorites': mode_favorites,
    }
    logger.debug('argv %s' % sys.argv)
    if mode in router:
        router[mode]()
    else:
        kodi_notification('Не реализованный режим работы %s' % mode)


if __name__ == '__main__':
    main()
